<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:ifc="http://www.buildingsmart-tech.org/ifcXML/IFC4/Add1" xmlns:msxsl="urn:schemas-microsoft-com:xslt" exclude-result-prefixes="xsl ifc msxsl">
	<xsl:output method="xml" omit-xml-declaration="yes"/>
	<xsl:variable name="LF" select="'&#xA;'"/>
	<xsl:template name="KeyPoolBuilder">
		<xsl:param name="ElementName"/>
		<xsl:variable name="stripedQuotesCSV">
			<xsl:call-template name="remove">
				<xsl:with-param name="input" select="$GetMappingTable"/>
				<xsl:with-param name="removal" select="'&quot;'"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:call-template name="csvrowfilter">
			<xsl:with-param name="localname" select="$ElementName"/>
			<xsl:with-param name="csvStringToFilter" select="$stripedQuotesCSV"/>
		</xsl:call-template>
	</xsl:template>
	<xsl:template name="csvrowfilter">
		<xsl:param name="localname" select="''"/>
		<xsl:param name="csvStringToFilter" select="''"/>
		<xsl:param name="currentElem" select="''"/>
		<xsl:choose>
			<xsl:when test="contains($csvStringToFilter,$LF)">
				<xsl:if test="contains(substring-before($csvStringToFilter,$LF),$localname)">
					<xsl:variable name="MappingCondition">
						<xsl:call-template name="csvcolumfilter">
							<xsl:with-param name="localname" select="$localname"/>
							<xsl:with-param name="csvStringToFilter" select="substring-before($csvStringToFilter,$LF)"/>
							<xsl:with-param name="getSingleSnippet" select="'mappingcondition'"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:choose>
						<xsl:when test="contains($MappingCondition, concat($localname, '.'))">
							<xsl:choose>
								<xsl:when test="contains($MappingCondition, '#')">
									<xsl:variable name="conditionalTerm" select="substring-after($MappingCondition, '###.###.')"/>
									<xsl:if test="contains($conditionalTerm, '.')">
										<xsl:choose>
											<xsl:when test="contains(substring-after($conditionalTerm,'.'), '.')">
												<xsl:variable name="preEvaluatePsets">
													<xsl:call-template name="GenerateTargetElementContentsFromSourceProperties">
														<xsl:with-param name="KeyPool" select="concat(concat(concat($conditionalTerm,' '),'SimDummy.ElementName'),'§')"/>
														<xsl:with-param name="preEvaluation" select="$conditionalTerm"/>
													</xsl:call-template>
												</xsl:variable>
												<xsl:if test="$preEvaluatePsets = 'TRUE'">
													<xsl:call-template name="csvcolumfilter">
														<xsl:with-param name="localname" select="$localname"/>
														<xsl:with-param name="csvStringToFilter" select="substring-before($csvStringToFilter,$LF)"/>
													</xsl:call-template>
												</xsl:if>
											</xsl:when>
											<xsl:otherwise>
												<xsl:variable name="fieldName" select="substring-before($conditionalTerm,'.')"/>
												<xsl:variable name="expectedValue">
													<xsl:call-template name="ToLower">
														<xsl:with-param name="input" select="substring-after($conditionalTerm,'.')"/>
													</xsl:call-template>
												</xsl:variable>
												<xsl:variable name="currentValue">
													<xsl:call-template name="ToLower">
														<xsl:with-param name="input" select="@*[name() = $fieldName or $localname = $fieldName]"/>
													</xsl:call-template>
												</xsl:variable>
												<xsl:if test="$currentValue = $expectedValue">
													<xsl:call-template name="csvcolumfilter">
														<xsl:with-param name="localname" select="$localname"/>
														<xsl:with-param name="csvStringToFilter" select="substring-before($csvStringToFilter,$LF)"/>
													</xsl:call-template>
												</xsl:if>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:if>
								</xsl:when>
								<xsl:when test="contains($MappingCondition, 'PredefinedType')">
									<xsl:variable name="expectedValue">
										<xsl:call-template name="ToLower">
											<xsl:with-param name="input" select="substring-after($MappingCondition, 'PredefinedType.')"/>
										</xsl:call-template>
									</xsl:variable>
									<xsl:choose>
										<xsl:when test="current()/@PredefinedType | current()/*/@PredefinedType">
											<xsl:if test="current()[@PredefinedType = $expectedValue] | current()/*[@PredefinedType = $expectedValue]">
												<xsl:call-template name="csvcolumfilter">
													<xsl:with-param name="localname" select="$localname"/>
													<xsl:with-param name="csvStringToFilter" select="substring-before($csvStringToFilter,$LF)"/>
												</xsl:call-template>
											</xsl:if>
										</xsl:when>
										<xsl:otherwise>
											<xsl:variable name="objectsType">
												<xsl:call-template name="IfcType">
													<xsl:with-param name="returnTypeObject" select="@ObjectType"/>
												</xsl:call-template>
											</xsl:variable>
											<xsl:if test="$objectsType = $expectedValue">
												<xsl:call-template name="csvcolumfilter">
													<xsl:with-param name="localname" select="$localname"/>
													<xsl:with-param name="csvStringToFilter" select="substring-before($csvStringToFilter,$LF)"/>
												</xsl:call-template>
											</xsl:if>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise>
							<xsl:call-template name="csvcolumfilter">
								<xsl:with-param name="localname" select="$localname"/>
								<xsl:with-param name="csvStringToFilter" select="substring-before($csvStringToFilter,$LF)"/>
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
				<xsl:call-template name="csvrowfilter">
					<xsl:with-param name="localname" select="$localname"/>
					<xsl:with-param name="csvStringToFilter">
						<xsl:value-of select="substring-after($csvStringToFilter,$LF)"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="csvcolumfilter">
		<xsl:param name="localname" select="''"/>
		<xsl:param name="csvStringToFilter" select="''"/>
		<xsl:param name="getSingleSnippet" select="''"/>
		<xsl:choose>
			<xsl:when test="contains($csvStringToFilter,';')">
				<xsl:variable name="preview2delim" select="substring-after(substring-after($csvStringToFilter, ';'), ';')"/>
				<xsl:if test="starts-with($preview2delim, 'Sim')">
					<xsl:variable name="mapConditionColumn">
						<xsl:value-of select="substring-before($csvStringToFilter,';')"/>
					</xsl:variable>
					<xsl:choose>
						<xsl:when test="$getSingleSnippet != ''">
							<xsl:value-of select="$mapConditionColumn"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:variable name="targetColumn">
								<xsl:value-of select="substring-before($preview2delim,';')"/>
							</xsl:variable>
							<xsl:variable name="sourceColumn">
								<xsl:value-of select="substring-before(substring-after($csvStringToFilter, ';'),';')"/>
							</xsl:variable>
							<xsl:value-of select="concat($mapConditionColumn, ' ')"/>
							<xsl:value-of select="concat($sourceColumn, ' ')"/>
							<xsl:value-of select="concat($targetColumn, '§')"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
				<xsl:call-template name="csvcolumfilter">
					<xsl:with-param name="localname" select="$localname"/>
					<xsl:with-param name="csvStringToFilter">
						<xsl:value-of select="substring-after($csvStringToFilter,';')"/>
					</xsl:with-param>
					<xsl:with-param name="getSingleSnippet" select="$getSingleSnippet"/>
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
	</xsl:template>
</xsl:stylesheet>
